@@ -1,3 +1,3 @@
-from routing_transformer.routing_transformer import RoutingTransformer, RoutingTransformerLM
+from routing_transformer.routing_transformer import RoutingTransformer, RoutingTransformerLM, update_kmeans_on_backwards
 from routing_transformer.autoregressive_wrapper import AutoregressiveWrapper
 from routing_transformer.autopadder import Autopadder
