@@ -2,6 +2,7 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import math
+from inspect import isfunction
 from operator import mul
 from functools import partial, reduce
 from routing_transformer.reversible import ReversibleSequence, SequentialSequence
@@ -16,8 +17,13 @@ KMEAN_INIT_ITERS = 10
 def identity(x, *args, **kwargs):
     return x
 
-def default(val, default_val):
-    return default_val if val is None else val
+def default(x, d):
+    if x is None:
+        return d if not isfunction(d) else d()
+    return x
+
+def to(t):
+    return {'device': t.device, 'dtype': t.dtype}
 
 def find_modules(nn_module, type):
     return [module for module in nn_module.modules() if isinstance(module, type)]
@@ -25,92 +31,28 @@ def find_modules(nn_module, type):
 def is_empty(t):
     return t.nelement() == 0
 
-def ema_inplace(moving_avg, new, decay):
-    if is_empty(moving_avg):
-        moving_avg.data.copy_(new)
-        return
-    moving_avg.data.mul_(decay).add_(1 - decay, new)
-
 def max_neg_value(tensor):
     return -torch.finfo(tensor.dtype).max
 
-def to(t):
-    return {'device': t.device, 'dtype': t.dtype}
-
 def batched_index_select(values, indices):
     last_dim = values.shape[-1]
-    return values.gather(2, indices[:, :, :, None].expand(-1, -1, -1, last_dim))
+    return values.gather(2, expand_dim(indices, -1, last_dim))
 
-def batched_bincount(index, num_classes, dim=-1):
-    shape = list(index.shape)
-    shape[dim] = num_classes
-    out = index.new_zeros(shape)
-    out.scatter_add_(dim, index, torch.ones_like(index, dtype=index.dtype))
-    return out
+def merge_dims(ind_from, ind_to, tensor):
+    shape = list(tensor.shape)
+    arr_slice = slice(ind_from, ind_to + 1)
+    shape[arr_slice] = [reduce(mul, shape[arr_slice])]
+    return tensor.reshape(*shape)
 
-def similarity(x, means):
-    return torch.einsum('bhld,hcd->bhlc', x, means)
+def expand_dim(t, dim, k):
+    t = t.unsqueeze(dim)
+    expand_shape = [-1] * len(t.shape)
+    expand_shape[dim] = k
+    return t.expand(*expand_shape)
 
-def squared_distance(x, means):
-    return 2 - 2 * similarity(x, means)
-
-def dists_and_buckets(x, means):
-    dists = similarity(x, means)
-    dists, buckets = torch.max(dists, dim=-1)
-    return dists, buckets
-
-def compute_se(x, means):
-    dists = squared_distance(x, means)
-    dists, buckets = torch.min(dists, dim=-1)
-    dists_dtype = dists.dtype
-    dists_float = dists.float()
-    return dists.sum(-1).type(dists_dtype)
-
-def buckets_to_means(x, buckets, num_clusters):
-    b, h, l, d = x.shape
-    means = buckets.new_zeros(b, h, num_clusters, d).float()  # hcd
-    means.scatter_add_(-2, buckets.unsqueeze(-1).expand(-1, -1, -1, d), x.float())
-    return F.normalize(means.sum(0, keepdim=True).type(x.dtype), dim=-1)
-
-def kmeans_iter(x, means, num_clusters, compute_se_):  # hard k-means single iter
-    dists, buckets = dists_and_buckets(x, means)
-    bins = batched_bincount(buckets, num_clusters).sum(0, keepdim=True)
-    zero_mask = bins.long() == 0
-    means_ = buckets_to_means(x, buckets, num_clusters)
-    means = torch.where(zero_mask.unsqueeze(-1), means, means_)
-    means = means.squeeze(0)
-    se = compute_se(x, means) if compute_se_ else None
-    return means, se
-
-def kmeans(x, means, training=True, init=False, compute_se=False):
-    b, *_, d = x.shape
-    num_clusters = means.shape[1]
-
-    max_iters = 1 if training else 0
-    
-    if init:
-        max_iters = max(KMEAN_INIT_ITERS, max_iters)
-        means = x.transpose(0, 1).contiguous().view(t, -1, d)
-        means = means[:, torch.randperm(num_clusters, device=x.device)[:num_clusters]]
-
-    se = None
-
-    for idx in range(max_iters):
-        means, se = kmeans_iter(x, means, num_clusters, compute_se)
-
-    dists = similarity(x, means)
-    return means, dists, se
-
-# pick the k points closest to each centroid and sort their indices in an ascending order for causal attention
-def distribution(dists, window_size):
-    _, topk_indices = dists.topk(k=window_size, dim=-2)
-    sort_val, _ = topk_indices.sort(dim=-2)
-    indices = sort_val.transpose(-2, -1)  # [b, h, c, l]
-    return indices
-
-def scatter_mean(t, index, dim, eps = 1e-5):
-    numer = torch.zeros_like(t).scatter_add(dim, index, t)
-    denom = torch.zeros_like(t).scatter_add(dim, index, torch.ones_like(t))
+def scatter_mean(src, t, index, dim, eps = 1e-5):
+    numer = src.scatter_add(dim, index, t)
+    denom = src.scatter_add(dim, index, torch.ones_like(t))
     return numer / (denom + eps)
 
 def look_around(x, backward = 1, forward = 0, pad_value = -1, dim = 2):
@@ -126,6 +68,17 @@ def split_at_index(dim, index, t):
     r = (*pre_slices, slice(index, None))
     return t[l], t[r]
 
+def ema(old, new, decay):
+    if old is None:
+        return new
+    return old * decay + new * (1 - decay)
+
+def ema_inplace(moving_avg, new, decay):
+    if is_empty(moving_avg):
+        moving_avg.data.copy_(new)
+        return
+    moving_avg.data.mul_(decay).add_(1 - decay, new)
+
 # helper classes
 
 class Chunk(nn.Module):
@@ -146,9 +99,22 @@ class PreNorm(nn.ModuleList):
         super().__init__()
         self.norm = nn.LayerNorm(dim)
         self.fn = fn
-    def forward(self, x):
+    def forward(self, x, **kwargs):
         x = self.norm(x)
-        return self.fn(x)
+        return self.fn(x, **kwargs)
+
+class ProjectInOut(nn.Module):
+    def __init__(self, fn, dim_in, dim_out, project_out = True):
+        super().__init__()
+        self.fn = fn
+        self.project_in = nn.Linear(dim_in, dim_out)
+        self.project_out = nn.Linear(dim_out, dim_in) if project_out else identity
+
+    def forward(self, x, **kwargs):
+        x = self.project_in(x)
+        x, loss = self.fn(x, **kwargs)
+        x = self.project_out(x)
+        return x, loss
 
 # positional embeddings
 
@@ -230,6 +196,30 @@ class ParameterList(object):
     def to_list(self):
         return [getattr(self.kls, self._keyname(self.prefix, i)) for i in range(self.length)]
 
+# full attention
+
+class FullAttention(nn.Module):
+    def __init__(self, causal = False, dropout = 0.):
+        super().__init__()
+        self.causal = causal
+        self.dropout = nn.Dropout(dropout)
+
+    def forward(self, q, k, v, input_mask = None, **kwargs):
+        b, h, t, d = q.shape
+        dot = torch.einsum('bhie,bhje->bhij', q, k) * (d ** -0.5)
+
+        masked_value = max_neg_value(dot)
+
+        if self.causal:
+            i, j = torch.triu_indices(t, t, 1)
+            dot[:, :, i, j] = masked_value
+
+        dot = dot.softmax(dim=-1)
+        dot = self.dropout(dot)
+
+        out = torch.einsum('bhij,bhje->bhie', dot, v)
+        return out
+
 # local attention
 
 class LocalAttention(nn.Module):
@@ -312,95 +302,190 @@ class LocalAttention(nn.Module):
         out = out.reshape(*shape)
         return out
 
-# kmeans attention
+# kmeans related function and class
 
-def register_kmeans_update_on_backwards(module):
-    module.kmean_attention_modules = find_modules(module, KmeansAttention)
+def update_kmeans_on_backwards(module):
+    module.kmean_modules = find_modules(module, Kmeans)
     def hook(_, grad_in, grad_out):
-        for m in module.kmean_attention_modules:
-            m.update_kmeans()
+        for m in module.kmean_modules:
+            m.update()
 
     return module.register_backward_hook(hook)
 
+def similarity(x, means):
+    return torch.einsum('bhld,hcd->bhlc', x, means)
+
+def dists_and_buckets(x, means):
+    dists = similarity(x, means)
+    _, buckets = torch.max(dists, dim=-1)
+    return dists, buckets
+
+def batched_bincount(index, num_classes, dim=-1):
+    shape = list(index.shape)
+    shape[dim] = num_classes
+    out = index.new_zeros(shape)
+    out.scatter_add_(dim, index, torch.ones_like(index, dtype=index.dtype))
+    return out
+
+def buckets_to_means(x, buckets, num_clusters):
+    b, h, l, d, dtype = *x.shape, x.dtype
+    means = buckets.new_zeros(b, h, num_clusters, d, dtype=dtype)
+    means.scatter_add_(-2, expand_dim(buckets, -1, d), x)
+    return F.normalize(means.sum(0, keepdim=True), dim=-1).type(dtype)
+
+def kmeans_iter(x, means):
+    num_clusters = means.shape[1]
+    dists, buckets = dists_and_buckets(x, means)
+    bins = batched_bincount(buckets, num_clusters).sum(0, keepdim=True)
+    zero_mask = bins.long() == 0
+    means_ = buckets_to_means(x, buckets, num_clusters)
+    means = torch.where(zero_mask.unsqueeze(-1), means, means_)
+    means = means.squeeze(0)
+    return means, buckets, dists
+
+def kmeans(x, means, training=True, init=False):
+    b, h, t, d = x.shape
+    max_iters = 1 if training else 0
+
+    if init:
+        num_clusters = means.shape[1]
+        max_iters = max(KMEAN_INIT_ITERS, max_iters)
+        means = x.transpose(0, 1).contiguous().view(h, -1, d)
+        indices = torch.randperm(means.size(1), device=x.device)[:num_clusters]
+        means = means[:, indices]
+
+    for idx in range(max_iters):
+        means, buckets, dists = kmeans_iter(x, means)
+
+    if max_iters == 0:
+        dists, buckets = dists_and_buckets(x, means)
+
+    return means, buckets, dists
+
+def distribution(dists, window_size):
+    _, topk_indices = dists.topk(k=window_size, dim=-2)
+    sort_val, _ = topk_indices.sort(dim=-2)
+    indices = sort_val.transpose(-2, -1)
+    return indices
+
+class Kmeans(nn.Module):
+    def __init__(self, num_heads, head_dim, num_clusters, ema_decay = 0.999, commitment = 1e-4):
+        super().__init__()
+        self.commitment = commitment
+        self.ema_decay = ema_decay
+
+        self.register_buffer('means', torch.randn(num_heads, num_clusters, head_dim))
+        self.register_buffer('initted', torch.tensor(False))
+        self.new_means = None
+
+    def update(self, new_means = None):
+        new_means = default(new_means, self.new_means)
+        assert new_means is not None, 'new kmeans has not been supplied'
+
+        first = not self.initted
+        ema_inplace(self.means, new_means, 0. if first else self.ema_decay)
+
+        if first:
+            self.initted = torch.tensor(True)
+
+        del self.new_means
+        self.new_means = None
+
+    def forward(self, x, window_size):
+        b, dtype = x.shape[0], x.dtype
+        means = self.means.type(dtype)
+        x = F.normalize(x, 2, dim=-1).type(dtype)
+
+        with torch.no_grad():
+            means, buckets, dists = kmeans(x, means, training=self.training, init=not self.initted)
+            indices = distribution(dists, window_size)
+            indices = indices.contiguous().view(*indices.size()[:2], -1)
+
+        routed_means = batched_index_select(expand_dim(means, 0, b), buckets)
+        loss = F.mse_loss(x, routed_means) * self.commitment
+
+        if self.training:
+            self.new_means = ema(self.new_means, means, 0.5)
+
+        return indices, loss
+
+# kmeans attention class
+
 class KmeansAttention(nn.Module):
-    def __init__(self, num_clusters, window_size, num_heads, head_dim, causal = False, dropout = 0., ema_decay = 0.999):
+    def __init__(self, num_clusters, window_size, num_heads, head_dim, causal = False, dropout = 0., ema_decay = 0.999, commitment = 1e-4, context_window_size = None, shared_qk = True):
         super().__init__()
         self.num_heads = num_heads
         self.num_clusters = num_clusters
         self.head_dim = head_dim
 
         self.window_size = window_size
+        self.context_window_size = default(context_window_size, window_size)
         self.causal = causal
 
-        self.router = nn.Parameter(torch.randn(num_heads, head_dim, head_dim))
-
-        self.rel_pos = RelativePositionalEmbedding(head_dim, num_heads, window_size)
-
-        self.ema_decay = ema_decay
-        self.register_buffer('means', torch.zeros(num_heads, num_clusters, head_dim))
-        self.register_buffer('new_means', torch.empty(num_heads, num_clusters, head_dim))
-        self.register_buffer('initted', torch.tensor(True))
-
+        self.shared_qk = shared_qk
+        self.kmeans = Kmeans(num_heads, head_dim, num_clusters, ema_decay, commitment)
         self.dropout = nn.Dropout(dropout)
 
-    def update_kmeans(self, new_means = None):
-        new_means = default(new_means, self.new_means)
-        assert not is_empty(new_means), 'new kmeans has not been supplied'
+    def forward(self, q, k, v, query_mask = None, key_mask = None):
+        b, h, t, d, kv_t, wsz, c_wsz, nc, device, dtype = *q.shape, k.shape[2], self.window_size, self.context_window_size, self.num_clusters, q.device, q.dtype
+        out = torch.zeros_like(q, dtype=dtype)
 
-        first = not self.initted
-        ema_inplace(self.means, new_means, 0. if first else self.ema_decay)
+        reshape_with_window = lambda x: x.reshape(b, h, nc, -1, d)
 
-        if first:
-            self.initted[0] = True
+        if self.shared_qk:
+            wsz = min(wsz, t)
+            indices, aux_loss = self.kmeans(k, wsz)
+            k = F.normalize(k, 2, dim=-1).type(dtype)
+            key_mask = query_mask
+            kv_indices = indices
+        else:
+            wsz = min(wsz, t)
+            c_wsz = min(c_wsz, kv_t)
 
-        if not is_empty(self.new_means):
-            self.new_means = torch.empty_like(self.new_means)
+            if wsz < self.window_size:
+                c_wsz = (self.window_size * self.context_window_size) // wsz
+            indices, loss_1 = self.kmeans(q, wsz)
+            kv_indices, loss_2 = self.kmeans(k, c_wsz)
+            aux_loss = loss_1 + loss_2
 
-    def forward(self, qk, v, **kwargs):
-        b, h, t, d, wsz, device = *qk.shape, self.window_size, qk.device
-        num_clusters = t // wsz
+        q = batched_index_select(q, indices)
+        k = batched_index_select(k, kv_indices)
+        v = batched_index_select(v, kv_indices)
 
-        with torch.no_grad():
-            k_routing = torch.einsum('bhtd,hdr->bhtr', qk, self.router)
-            k_routing = F.normalize(qk, dim=-1)
-
-            means, dists, se = kmeans(k_routing, self.means, training=self.training, init=not self.initted)
-            indices = distribution(dists, wsz)
-
-            if self.training:
-                self.new_means.copy_(means)
-
-            indices = indices.contiguous().view(*indices.size()[:2], -1)
-        
-        qk = batched_index_select(qk, indices)
-        v = batched_index_select(v, indices)
-
-        qk, v = map(lambda x: x.reshape(b, h, num_clusters, wsz, d), (qk, v))
-
-        q = qk
-        k = F.normalize(qk, 2, dim=-1).type(qk.dtype)
+        q, k, v = map(reshape_with_window, (q, k, v))
 
         dots = torch.einsum('bhnid,bhnjd->bhnij', q, k) * (d ** -0.5)
-        dots = dots + self.rel_pos(q)
 
         mask_value = max_neg_value(dots)
 
+        if query_mask is not None or key_mask is not None:
+            query_mask = default(query_mask, lambda: torch.ones((b, t), device=device).bool())
+            key_mask = default(key_mask, lambda: torch.ones((b, kv_t), device=device).bool())
+
+            q_mask = expand_dim(query_mask, 1, h).gather(2, indices)
+            kv_mask = expand_dim(key_mask, 1, h).gather(2, kv_indices)
+            q_mask, kv_mask = map(lambda t: t.reshape(b, h, nc, -1), (q_mask, kv_mask))
+            mask = q_mask[:, :, :, :, None] * kv_mask[:, :, :, None, :]
+            dots.masked_fill_(~mask, mask_value)
+            del mask
+
         if self.causal:
             mask = torch.ones(wsz, wsz, device=device).byte().triu_(1).bool()
             dots.masked_fill_(mask, mask_value)
             del mask
 
-        mask = torch.eye(wsz, device=dots.device).bool()
-        dots.masked_fill_(mask, TOKEN_SELF_ATTN_VALUE)
-        del mask
+        if self.shared_qk:
+            mask = torch.eye(wsz, device=dots.device).bool()
+            dots.masked_fill_(mask, TOKEN_SELF_ATTN_VALUE)
+            del mask
 
         dots = dots.softmax(dim=-1)
         dots = self.dropout(dots)
 
         bo = torch.einsum('bhcij,bhcjd->bhcid', dots, v)
-        so = torch.reshape(bo, (b, h, -1, bo.shape[-1])).float()
-        out = scatter_mean(so, indices.unsqueeze(-1).expand_as(so), -2)
-        return out
+        so = torch.reshape(bo, (b, h, -1, bo.shape[-1])).type(dtype)
+        out = scatter_mean(out, so, indices.unsqueeze(-1).expand_as(so), -2)
+        return out, aux_loss
 
 # feedforward
 
@@ -436,59 +521,89 @@ class FeedForward(nn.Module):
 # self attention
 
 class SelfAttention(nn.Module):
-    def __init__(self,  dim, depth, max_seq_len, heads, local_attn_heads, window_size, causal = False, attn_dropout = 0., dropout = 0., kmeans_ema_decay = 0.999):
+    def __init__(self,  dim, depth, max_seq_len, heads, local_attn_heads, window_size, local_attn_window_size = None, causal = False, attn_dropout = 0., dropout = 0., kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None):
         super().__init__()
         assert (dim % heads) == 0, 'hidden dimension must be divisible by number of heads'
         assert (max_seq_len % window_size) == 0, 'maximum sequence length must be divisible by the target window size'
         assert local_attn_heads <= heads, 'number of local attention heads must be less than total heads'
+        assert not (receives_context and local_attn_heads > 0), 'local attention cannot be used for self attention with context'
+        assert not (receives_context and causal), 'contextual attention layer cannot be causal'
 
+        local_attn_window_size = default(local_attn_window_size, window_size // 2)
+        context_window_size = default(context_window_size, window_size)
+
+        self.receives_context = receives_context
         self.heads = heads
-
         self.local_attn_heads = local_attn_heads
         self.global_attn_heads = heads - local_attn_heads
 
-        self.local_attn = self.global_attn = None
+        self.window_size = window_size
 
         head_dim = dim // heads
         num_clusters = max_seq_len // window_size
 
         if self.local_attn_heads > 0:
-            self.local_attn = LocalAttention(window_size // 2, local_attn_heads, head_dim, causal = True, shared_qk = True, dropout = attn_dropout)
+            self.local_attn = LocalAttention(local_attn_window_size, local_attn_heads, head_dim, causal = True, shared_qk = True, dropout = attn_dropout)
 
         if self.global_attn_heads > 0:
-            self.global_attn = KmeansAttention(num_clusters, window_size, self.global_attn_heads, head_dim, causal = causal, dropout = attn_dropout, ema_decay = kmeans_ema_decay)
+            self.global_attn = KmeansAttention(num_clusters, window_size, self.global_attn_heads, head_dim, causal = causal, dropout = attn_dropout, ema_decay = kmeans_ema_decay, commitment = commitment_factor, shared_qk = not receives_context)
 
-        self.to_qkv = nn.Linear(dim, dim * 2, bias = False)
+        if self.receives_context:
+            self.full_attn = FullAttention(causal = causal, dropout = attn_dropout)
+
+        self.to_qk = nn.Linear(dim, dim, bias = False)
+        self.to_v = nn.Linear(dim, dim, bias = False)
         self.to_out = nn.Linear(dim, dim, bias = False)
 
         self.dropout = nn.Dropout(dropout)
 
-    def forward(self, x):
+    def forward(self, x, context = None, input_mask = None, context_mask = None):
+        assert not (self.receives_context and context is None), 'context must be passed if self attention is set to receive context'
         b, t, e, h = *x.shape, self.heads
+        head_dim = e // self.heads
 
-        qk, v = self.to_qkv(x).chunk(2, dim=-1)
-        split_heads = lambda v: v.reshape(b, t, h, -1).transpose(1, 2).contiguous()
-        qk, v = map(split_heads, (qk, v))
+        split_heads = lambda v: v.reshape(b, -1, h, head_dim).transpose(1, 2).contiguous()
 
-        split_index_fn = partial(split_at_index, 1, self.local_attn_heads)
-        (lqk, qk), (lv, v) = map(split_index_fn, (qk, v))
-        has_local, has_global = map(lambda x: x.shape[1] > 0, (lqk, qk))
+        if not self.receives_context:
+            qk, v = self.to_qk(x), self.to_v(x)
+
+            qk, v = map(split_heads, (qk, v))
+            split_index_fn = partial(split_at_index, 1, self.local_attn_heads)
+            (lqk, qk), (lv, v) = map(split_index_fn, (qk, v))
+            has_local, has_global = map(lambda x: x.shape[1] > 0, (lqk, qk))
+
+            q, k, v = qk, qk, v
+        else:
+            q, k, v = self.to_qk(x), self.to_qk(context), self.to_v(context)
+            q, k, v = map(split_heads, (q, k, v))
+            has_local, has_global = False, True
 
         out = []
+        total_loss = torch.tensor(0., **to(x)).requires_grad_()
+
         if has_local:
-            out.append(self.local_attn(lqk, lqk, lv))
+            local_out = self.local_attn(lqk, lqk, lv, input_mask = input_mask)
+            out.append(local_out)
 
         if has_global:
-            out.append(self.global_attn(qk, v))
+            global_out, loss = self.global_attn(q, k, v, query_mask = input_mask, key_mask = context_mask)
+            total_loss = total_loss + loss
+
+            if self.receives_context:
+                full_out = self.full_attn(q[:, :, 0:self.window_size], k, v)
+                global_out = torch.cat((full_out, global_out[:, :, self.window_size:]), dim=2)
+
+            out.append(global_out)
 
         out = torch.cat(out, dim=1)
         out = out.reshape(b, h, t, -1).transpose(1, 2).reshape(b, t, -1)
         out = self.to_out(out)
-        return self.dropout(out)
+        return self.dropout(out), total_loss
 
 class RoutingTransformer(nn.Module):
-    def __init__(self, dim, depth, max_seq_len, heads = 8, window_size = 64, causal = False, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., n_local_attn_heads = 0, ff_glu = False, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999):
+    def __init__(self, dim, depth, max_seq_len, heads = 8, window_size = 64, local_attn_window_size = None, causal = False, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., n_local_attn_heads = 0, ff_glu = False, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None):
         super().__init__()
+        local_attn_window_size = default(local_attn_window_size, window_size // 2)
         if type(n_local_attn_heads) is not tuple:
             n_local_attn_heads = tuple([n_local_attn_heads] * depth)
 
@@ -499,33 +614,52 @@ class RoutingTransformer(nn.Module):
         fn_wrapper = partial(PreNorm, dim)
 
         for ind, local_heads in zip(range(depth), n_local_attn_heads):
-            attn = SelfAttention(dim, depth, max_seq_len, heads, local_heads, window_size, causal = causal, attn_dropout = attn_dropout, dropout = attn_layer_dropout, kmeans_ema_decay = kmeans_ema_decay)
+            attn = SelfAttention(dim, depth, max_seq_len, heads, local_heads, window_size, causal = causal, local_attn_window_size = local_attn_window_size, attn_dropout = attn_dropout, dropout = attn_layer_dropout, kmeans_ema_decay = kmeans_ema_decay, commitment_factor = commitment_factor)
             ff = Chunk(ff_chunks, FeedForward(dim, dropout = ff_dropout, glu = ff_glu), along_dim=1)
 
             attn, ff = map(fn_wrapper, (attn, ff))
             layers.append(nn.ModuleList([attn, ff]))
 
-        execute_type = ReversibleSequence if reversible else SequentialSequence
-        self.layers = execute_type(layers, layer_dropout = layer_dropout)
-        self.pad_to_window_size = window_size
+            if not receives_context:
+                continue
 
-        self.remove_handle_fn = register_kmeans_update_on_backwards(self)        
+            context_attn = SelfAttention(dim, depth, max_seq_len, heads, 0, window_size, local_attn_window_size = local_attn_window_size, attn_dropout = attn_dropout, dropout = attn_layer_dropout, kmeans_ema_decay = kmeans_ema_decay, commitment_factor = commitment_factor, receives_context = True, context_window_size = context_window_size)
+            context_ff = Chunk(ff_chunks, FeedForward(dim, dropout = ff_dropout, glu = ff_glu), along_dim=1)
+            layers.append(nn.ModuleList([context_attn, context_ff]))
+
+        execute_type = ReversibleSequence if reversible else SequentialSequence
+
+        attn_context_layer = ((True, False),) if receives_context else tuple()
+        route_attn = ((True, False), *attn_context_layer) * depth
+        route_context = ((False, False), *attn_context_layer) * depth
+
+        context_route_map = {'context': route_context, 'context_mask': route_context} if receives_context else {}
+        attn_route_map = {'input_mask': route_attn}
+        self.layers = execute_type(layers, args_route = {**attn_route_map, **context_route_map}, layer_dropout = layer_dropout)
+
+        self.pad_to_window_size = local_attn_window_size
 
     def forward(self, x, **kwargs):
-        x = self.layers(x, **kwargs)
-        return x
+        x, loss = self.layers(x, **kwargs)
+        return x, loss
 
 class RoutingTransformerLM(nn.Module):
-    def __init__(self, num_tokens, dim, depth, max_seq_len, heads = 8, window_size = 64, causal = False, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., ff_mult = 4, ff_activation = None, ff_glu = False, return_embeddings = False, n_local_attn_heads = 0, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999):
+    def __init__(self, num_tokens, dim, depth, max_seq_len, heads = 8, window_size = 64, local_attn_window_size = None, causal = False, emb_dim = None, attn_dropout = 0., ff_dropout = 0., attn_layer_dropout = 0., layer_dropout = 0., ff_mult = 4, ff_activation = None, ff_glu = False, return_embeddings = False, n_local_attn_heads = 0, reversible = False, ff_chunks = 1, kmeans_ema_decay = 0.999, commitment_factor = 1e-4, receives_context = False, context_window_size = None):
         super().__init__()
+        emb_dim = default(emb_dim, dim)
         self.max_seq_len = max_seq_len
-        self.token_emb = nn.Embedding(num_tokens, dim)
-        self.axial_pos_emb = AxialPositionalEmbedding(dim, max_seq_len, axial_shape=(max_seq_len // window_size, window_size))
-        self.routing_transformer = RoutingTransformer(dim, depth, max_seq_len, heads = heads, window_size = window_size, causal = causal, ff_dropout = ff_dropout, attn_dropout = attn_dropout, attn_layer_dropout = attn_layer_dropout, layer_dropout = layer_dropout, n_local_attn_heads = n_local_attn_heads, ff_glu = ff_glu, reversible = reversible, ff_chunks = ff_chunks, kmeans_ema_decay = kmeans_ema_decay)
-        self.out = nn.Linear(dim, num_tokens) if not return_embeddings else identity
+
+        self.token_emb = nn.Embedding(num_tokens, emb_dim)
+        self.axial_pos_emb = AxialPositionalEmbedding(emb_dim, max_seq_len, axial_shape=(max_seq_len // window_size, window_size))
+        self.routing_transformer = RoutingTransformer(dim, depth, max_seq_len, heads = heads, window_size = window_size, local_attn_window_size = local_attn_window_size, causal = causal, ff_dropout = ff_dropout, attn_dropout = attn_dropout, attn_layer_dropout = attn_layer_dropout, layer_dropout = layer_dropout, n_local_attn_heads = n_local_attn_heads, ff_glu = ff_glu, reversible = reversible, ff_chunks = ff_chunks, kmeans_ema_decay = kmeans_ema_decay, receives_context = receives_context, context_window_size = context_window_size)
+
+        if emb_dim != dim:
+            self.routing_transformer = ProjectInOut(self.routing_transformer, emb_dim, dim, project_out = not return_embeddings)
+
+        self.out = nn.Linear(emb_dim, num_tokens) if not return_embeddings else identity
 
     def forward(self, x, **kwargs):
         x = self.token_emb(x)
         x = x + self.axial_pos_emb(x)
-        x = self.routing_transformer(x, **kwargs)
-        return self.out(x)
+        x, loss = self.routing_transformer(x, **kwargs)
+        return self.out(x), loss
